// This #include statement was automatically added by the Particle IDE.
#include <neopixel.h>

// This #include statement was automatically added by the Particle IDE.
#include <SharpDustSensor.h>

SharpDustSensor sensor = SharpDustSensor( 1, D7, A2 );
double density = 0;
int outsideLevels = 0;//this will be updated daily?
int densityLevel = 0; //note, this is used to categorize the "density" double into int values

//current inside color codes
int currentInsideR = 0;
int currentInsideG = 0;
int currentInsideB = 0;

//future inside color codes
int futureInsideR = 0;
int futureInsideG = 0;
int futureInsideB = 0;

//current outside color codes
int currentOutsideR = 0;
int currentOutsideG = 0;
int currentOutsideB = 0;

//future outside color codes
int futureOutsideR = 0;
int futureOutsideG = 0;
int futureOutsideB = 0;

//variables for the color shifting
int fadeRate = 50;
int steps = 70;


//D5 and D4 codes
#define PIXEL_COUNT1 7
#define PIXEL_TYPE1 WS2812
#define PIXEL_PIN1 D5
#define PIXEL_COUNT2 7
#define PIXEL_TYPE2 WS2812
#define PIXEL_PIN2 D4

Adafruit_NeoPixel insideStrip = Adafruit_NeoPixel( PIXEL_COUNT1, PIXEL_PIN1, PIXEL_TYPE1 );
Adafruit_NeoPixel outsideStrip = Adafruit_NeoPixel( PIXEL_COUNT2, PIXEL_PIN2, PIXEL_TYPE2 );



void setup() {
  sensor.begin();
  Particle.variable("dustDensity",density);
  Particle.function("OutsideLevel_Override",resetOutsideLevels);
  Particle.function("DustLevel_Override",enterDustLevels);
  insideStrip.begin();
  outsideStrip.begin();
  
  //varialbes for testing
  Particle.variable("currentB",currentInsideB);
  Particle.variable("futureB",futureInsideB);
  Particle.variable("dustLEVELS",densityLevel);
}

void loop() {
    //get the dust value (local environment. Note that external environment is updated from IFTTT as required)
    density = sensor.getDensity(); 
    
    
    //normalize the inputs
    if(density>=.11){
        densityLevel = 5;
    }
    else if(density>=.07){
        densityLevel = 4;
    }
    else if(density>=.04){
        densityLevel = 3;
    }
    else if(density>=.015){
        densityLevel = 2;
    }
    else if(density>=.005){
        densityLevel = 1;
    }
    else{
        densityLevel = 0;
    }
    
    
    if(outsideLevels>=10){
        outsideLevels = 10;
    }
    if(outsideLevels<=0.0){
        outsideLevels = 0;
    }

    //map the "future" colors
    futureInsideR = map(densityLevel,0,5,255,230);
    futureInsideG = map(densityLevel,0,5,230,60);
    futureInsideB = map(densityLevel,0,5,240,5);
    futureOutsideR = map(outsideLevels,0,10,255,230);
    futureOutsideG = map(outsideLevels,0,10,245,60);
    futureOutsideB = map(outsideLevels,0,10,245,5);
  
    //change lights from present to future
    fadeInside(currentInsideR, currentInsideG, currentInsideB, futureInsideR, futureInsideG, futureInsideB, fadeRate, steps);
    fadeOutside(currentOutsideR, currentOutsideG, currentOutsideB, futureOutsideR, futureOutsideG, futureOutsideB, fadeRate, steps);
    delay(50);
    
    //reassign "present"
    currentInsideR = futureInsideR;
    currentInsideG = futureInsideG;
    currentInsideB = futureInsideB;
    
    currentOutsideR = futureOutsideR;
    currentOutsideG = futureOutsideG;
    currentOutsideB = futureOutsideB;
    
    //delay to make the program chill out some
     delay(500);
}

//call this function every morning through IFTTT
int resetOutsideLevels(String PollenLevels){
    //update.
    outsideLevels = PollenLevels.toInt();
}

//this function is used to gently change lights
void fadeInside(int r1, int g1, int b1, int r2, int g2, int b2, int fadeRate, int steps) {
  for (int i = 1; i < steps; i++)
  {
    uint8_t red = (((r1 * (steps - i)) + (r2 * i)) / steps);
    uint8_t green = (((g1 * (steps - i)) + (g2 * i)) / steps);
    uint8_t blue = (((b1 * (steps - i)) + (b2 * i)) / steps);
    // Sets the pixels to the color adjusted in the fade
    for (int x = 0; x < 7; x++) {
      insideStrip.setPixelColor(x, red, green, blue);
    }
    insideStrip.show();
    delay(fadeRate);
  }
}

void fadeOutside(int r1, int g1, int b1, int r2, int g2, int b2, int fadeRate, int steps) {
  for (int i = 1; i < steps; i++)
  {
    uint8_t red2 = (((r1 * (steps - i)) + (r2 * i)) / steps);
    uint8_t green2 = (((g1 * (steps - i)) + (g2 * i)) / steps);
    uint8_t blue2 = (((b1 * (steps - i)) + (b2 * i)) / steps);
    // Sets the pixels to the color adjusted in the fade
    for (int x = 0; x < 7; x++) {
      outsideStrip.setPixelColor(x, red2, green2, blue2);
    }
    outsideStrip.show();
    delay(fadeRate);
  }
}

//this function strictly exists to manually change the inside dust levels for testing. (note, has to be between 0-5)
//THIS FUNCTION WILL NOT WORK IF LOOP IS UPDATING WITH THE REAL VALUES
int enterDustLevels(String dustLevelsManually){
    densityLevel = dustLevelsManually.toInt();
}
